@@ -60,8 +60,8 @@ void buttonPress(int button, int state, int x, int y);
 void buttonRelease(int button, int state, int x, int y);
 int PlayerConnect(void);
 void DisplayRoutine();
-void DrawPos (int pos);
-void DrawTime(int time);
+void DrawTrackNum (int track_num);
+void DrawTime(gint64 time);
 void DrawArrow(void);
 void DrawVolume(void);
 void DrawTitle(char *title);
@@ -598,7 +598,8 @@ int PlayerConnect(void)
 
 void DisplayRoutine()
 {
-	int time = 0, length = 0, position = 100;
+	gint64 position = 0, length = 0;
+	int track_num = 100;
 	char *title = NULL;
 	GError *error = NULL;
 
@@ -613,7 +614,7 @@ void DisplayRoutine()
 		title_pos = 0;
 		arrow_pos = 5;
 	} else {
-		char *length_str, *position_str;
+		char *length_str, *track_num_str;
 		PlayerctlPlaybackStatus status;
 
 		g_object_get(player, "playback-status", &status, NULL);
@@ -623,7 +624,11 @@ void DisplayRoutine()
 			    pause_norotate)
 				arrow_pos = 5;
 
-			g_object_get(player, "position", &time, NULL);
+			position = playerctl_player_get_position(player,
+								 &error);
+			if (error != NULL)
+				DAWarning("%s", error->message);
+			g_clear_error(&error);
 
 			title = playerctl_player_get_title(player,
 							   &error);
@@ -638,23 +643,24 @@ void DisplayRoutine()
 				DAWarning("%s", error->message);
 			g_clear_error(&error);
 			if (length_str) {
-				length = atoi(length_str);
+				length = g_ascii_strtoull(length_str,
+							  NULL, 10);
 				g_free(length_str);
 			} else
 				length = 0;
 
-			position_str =
+			track_num_str =
 				playerctl_player_print_metadata_prop(
 					player, "xesam:trackNumber",
 					&error);
 			if (error != NULL)
 				DAWarning("%s", error->message);
 			g_clear_error(&error);
-			if (position_str) {
-				position = atoi(position_str);
-				g_free(position_str);
+			if (track_num_str) {
+				track_num = atoi(track_num_str);
+				g_free(track_num_str);
 			} else
-				position = 0;
+				track_num = 0;
 		} else { /* not playing or paused */
 			title = strdup("--");
 			title_pos = 0;
@@ -663,9 +669,8 @@ void DisplayRoutine()
 	}
 
 	/*Draw everything */
-	if (t_time && length) DrawTime((length-time) / 1000);
-	else DrawTime(time / 1000);
-	DrawPos(position);
+	DrawTime(t_time && length ? length - position : position);
+	DrawTrackNum(track_num);
 	DrawArrow();
 	DrawVolume();
 	DrawTitle(title);
@@ -676,14 +681,15 @@ void DisplayRoutine()
 		free(title);
 }
 
-void DrawPos (int pos)
+void DrawTrackNum (int track_num)
 {
-	char posstr[16];
-	char *p = posstr;
+	char track_num_str[16];
+	char *p = track_num_str;
 	int i=1;
 
-	if (pos > 99) pos=0;
-	sprintf(posstr, "%02d", pos);
+	if (track_num > 99)
+		track_num = 0;
+	sprintf(track_num_str, "%02d", track_num);
 
 	for (;i<3; i++)
 	{
@@ -692,13 +698,13 @@ void DrawPos (int pos)
 	}
 }
 
-void DrawTime(int time)
+void DrawTime(gint64 time)
 {
 	char timestr[16];
 	char *p = timestr;
 	int i=0;
 
-	time = time / 1000;
+	time /= 1000000;
 
 	/* 3 cases:
 	 *     up to 99 minutes and 59 seconds
@@ -707,16 +713,18 @@ void DrawTime(int time)
 	 */
 	if (time < 6000)
 	{
-		sprintf(timestr, "%02d%02d", time / 60, time % 60);
-	} else {
-		if (time < 360000)
-		{
-			sprintf(timestr, "%02d%02d", time / 3600,
-					time % 3600 / 60);
-		} else {
-			sprintf(timestr, "%02d%02d", 0, 0);
-		}
+		int m = time / 60, s = time % 60;
+
+		sprintf(timestr, "%02d%02d", m, s);
 	}
+	else if (time < 360000)
+	{
+		int h = time / 3600, m = time % 3600;
+
+		sprintf(timestr, "%02d%02d", h, m);
+	}
+	else
+		strcpy (timestr, "0000");
 
 	for (;i<4; i++)
 	{
